<div id="container">
  <div id="header-container">
    <img src="/images/merb.jpg">
    <!-- <h1>Mongrel + Erb</h1> -->
    <h2>pocket rocket web framework</h2>
    <hr />
  </div>

  <div id="left-container">
    <h3>Exception:</h3>
    <p><%= request.exceptions.first.message %></p>
  </div>

  <div id="main-container">
    <h3>Welcome to Merb!</h3>
    <p>Merb is a light-weight MVC framework written in Ruby.  We hope you enjoy it.</p>
    
    <h3>Where can I find help?</h3>
    <p>If you have any questions or if you can't figure something out, please take a
      look at our <a href="http://merb.devjavu.com/"> project development page</a> or,
      feel free to come chat at irc.freenode.net, channel #merb.</p>

    <h3>How do I edit this page?</h3>
    <p>You're seeing this page because you need to edit the following files:
      <ul>
        <li>config/merb.yml <strong><em>(optional)</em></strong></li>
        <li>config/router.rb <strong><em>(recommended)</em></strong></li>
        <li>app/views/exceptions/not_found.html.erb <strong><em>(recommended)</em></strong></li>
        <li>app/views/layout/application.html.erb <strong><em>(change this layout)</em></strong></li>
      </ul>
    </p>
  </div>

  <div id="footer-container">
    <hr />
    <div class="left"></div>
    <div class="right">&copy; 2008 the merb dev team</div>
    <p>&nbsp;</p>
  </div>
</div>
