<div id="container">
  <div id="header-container">
    <img src="/images/merb.jpg">
    <!-- <h1>Mongrel + Erb</h1> -->
    <h2>pocket rocket web framework</h2>
    <hr />
  </div>

  <div id="left-container">
    <h3>Exception:</h3>
    <p><%= request.exceptions.first.message %></p>
  </div>

  <div id="main-container">
    <h3>Why am I seeing this page?</h3>
    <p>Merb couldn't find an appropriate content_type to return,
      based on what you said was available via provides() and
      what the client requested.  For more information, visit
      http://merbivore.com/fixing_406_issues
    </p> 
    
    <h3>Where can I find help?</h3>
    <p>If you have any questions or if you can't figure something out, please take a
      look at our <a href="http://merb.devjavu.com/"> project development page</a> or,
      feel free to come chat at irc.freenode.net, channel #merb.</p>

    <h3>How do I edit this page?</h3>
    <p>You can change what people see when this happens by editing <tt>app/views/exceptions/not_found.html.erb</tt>.</p> 
  
  </div>

  <div id="footer-container">
    <hr />
    <div class="left"></div>
    <div class="right">&copy; 2008 the merb dev team</div>
    <p>&nbsp;</p>
  </div>
</div>
