<%= form_for @obj, :builder => MyBuilder do %>
  <%= file_field(:foo) %>
  <%= text_field(:foo) %>
  <%= hidden_field(:foo) %>
  <%= password_field(:foo) %>
  <%= radio_button(:foo) %>
  <%= text_area(:foo) %>
  <%= button("Click") %>
  <%= submit("Awesome") %>
<% end =%>
