<%= check_box(:name => "nil", :value => nil) %>
<%= check_box(:name => "false", :value => false) %>
<%= check_box(:name => "0", :value => 0) %>
<%= check_box(:name => "0s", :value => "0") %>
<%= check_box(:name => "1", :value => 1) %>
<%= check_box(:name => "1s", :value => "1") %>
<%= check_box(:name => "true", :value => true) %>

